@@ -1,8 +1,7 @@
-//-*- C++ -*-
 /*
  * lingot, a musical instrument tuner.
  *
- * Copyright (C) 2004-2007  Ibán Cereijo Graña, Jairo Chapela Martínez.
+ * Copyright (C) 2004-2010  Ibán Cereijo Graña, Jairo Chapela Martínez.
  *
  * This file is part of lingot.
  *
@@ -15,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with lingot; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -35,12 +34,14 @@
 #include "lingot-gauge.h"
 #include "lingot-i18n.h"
 
+#include "lingot-error.h"
+
 #include "lingot-background.xpm"
 #include "lingot-logo.xpm"
 
 void lingot_mainframe_redraw(LingotMainFrame*);
-void lingot_mainframe_put_frequency(LingotMainFrame*);
-void lingot_mainframe_draw_gauge(LingotMainFrame*);
+void lingot_mainframe_filter_frequency_value(LingotMainFrame*);
+void lingot_mainframe_draw_gauge_and_labels(LingotMainFrame*);
 void lingot_mainframe_draw_spectrum(LingotMainFrame*);
 
 GdkColor black_color;
@@ -58,7 +59,7 @@ int gauge_size_y = 100;
 
 int spectrum_size_y = 64;
 
-// spectrum area margins 
+// spectrum area margins
 int spectrum_bottom_margin = 16;
 int spectrum_top_margin = 12;
 int spectrum_x_margin = 15;
@@ -70,38 +71,36 @@ PangoFontDescription* spectrum_legend_font_desc;
 
 void lingot_mainframe_callback_redraw(GtkWidget* w, GdkEventExpose* e,
 		LingotMainFrame* frame) {
-	if (frame->core->running)
-		lingot_mainframe_redraw(frame);
+	lingot_mainframe_redraw(frame);
 }
 
 void lingot_mainframe_callback_destroy(GtkWidget* w, LingotMainFrame* frame) {
-	if (frame->core->running) {
-		g_source_remove(frame->visualization_timer_uid);
-		g_source_remove(frame->calculation_timer_uid);
-		g_source_remove(frame->freq_timer_uid);
-		gtk_main_quit();
-	}
+	g_source_remove(frame->visualization_timer_uid);
+	g_source_remove(frame->calculation_timer_uid);
+	g_source_remove(frame->freq_timer_uid);
+	gtk_main_quit();
 }
 
 void lingot_mainframe_callback_about(GtkWidget* w, LingotMainFrame* frame) {
-	if (frame->core->running) {
-		static const gchar* authors[] = {
-				"Ibán Cereijo Graña <ibancg@gmail.com>",
-				"Jairo Chapela Martínez <jairochapela@gmail.com>", NULL };
+	static const gchar* authors[] = { "Ibán Cereijo Graña <ibancg@gmail.com>",
+			"Jairo Chapela Martínez <jairochapela@gmail.com>", NULL };
 
-		const gchar* artists[] = { "Matthew Blissett (Logo design)", NULL };
+	const gchar* artists[] = { "Matthew Blissett (Logo design)", NULL };
 
-		gtk_show_about_dialog(NULL,
-		"name", "Lingot",
-		"version", VERSION,
-		"copyright", "\xC2\xA9 2004-2007 Ibán Cereijo Graña\n\xC2\xA9 2004-2007 Jairo Chapela Martínez",
-		"comments", _("Accurate and easy to use musical instrument tuner"),
-		"authors", authors,
-		"artists", artists,
-		"translator-credits", _("translator-credits"),
-		"logo-icon-name", "lingot-logo",
-		NULL);
-	}
+	gtk_show_about_dialog(
+			NULL,
+			"name",
+			"Lingot",
+			"version",
+			VERSION,
+			"copyright",
+			"\xC2\xA9 2004-2010 Ibán Cereijo Graña\n\xC2\xA9 2004-2010 Jairo Chapela Martínez",
+			"comments", _("Accurate and easy to use musical instrument tuner"),
+			"authors", authors, "artists", artists, "website-label",
+			"http://lingot.nongnu.org/", "website",
+			"http://lingot.nongnu.org/", "translator-credits",
+			_("translator-credits"), "logo-icon-name", "lingot-logo",
+			NULL);
 }
 
 void lingot_mainframe_callback_view_spectrum(GtkWidget* w,
@@ -115,60 +114,74 @@ void lingot_mainframe_callback_view_spectrum(GtkWidget* w,
 
 void lingot_mainframe_callback_config_dialog(GtkWidget* w,
 		LingotMainFrame* frame) {
-	if (frame->core->running) {
-		if (!frame->config_dialog) {
-			frame->config_dialog = lingot_config_dialog_new(frame);
-		} else {
-			// TODO: activate config dialog
-		}
-	}
+	lingot_config_dialog_show(frame);
 }
 
-/* Callback for visualization */
-gboolean lingot_mainframe_callback_visualization(gpointer data) {
+/* timeout for gauge and labels visualization */
+gboolean lingot_mainframe_callback_tout_gauge_display(gpointer data) {
 	unsigned int period;
 
 	LingotMainFrame* frame = (LingotMainFrame*) data;
 
-	if (frame->core->running) {
-		lingot_mainframe_draw_gauge(frame);
+	period = 1000 / frame->conf->visualization_rate;
+	frame->visualization_timer_uid = g_timeout_add(period,
+			lingot_mainframe_callback_tout_gauge_display, frame);
 
-		period = 1000/frame->conf->visualization_rate;
-		frame->visualization_timer_uid = g_timeout_add(period,
-				lingot_mainframe_callback_visualization, frame);
-	}
+	lingot_mainframe_draw_gauge_and_labels(frame);
 
 	return 0;
 }
 
-/* Callback for calculation */
-gboolean lingot_mainframe_callback_calculation(gpointer data) {
+/* timeout for spectrum computation and display */
+gboolean lingot_mainframe_callback_tout_spectrum_computation_display(
+		gpointer data) {
 	unsigned int period;
 
 	LingotMainFrame* frame = (LingotMainFrame*) data;
 
-	if (frame->core->running) {
-		lingot_mainframe_draw_spectrum(frame);
+	period = 1000 / frame->conf->calculation_rate;
+	frame->calculation_timer_uid = g_timeout_add(period,
+			lingot_mainframe_callback_tout_spectrum_computation_display, frame);
 
-		period = 1000/frame->conf->calculation_rate;
-		frame->calculation_timer_uid = g_timeout_add(period,
-				lingot_mainframe_callback_calculation, frame);
-	}
+	lingot_mainframe_draw_spectrum(frame);
+	//lingot_core_compute_fundamental_fequency(frame->core);
 
 	return 0;
 }
 
-/* Callback for frequency calculation */
-gboolean lingot_mainframe_callback_frequency(gpointer data) {
+/* timeout for a new gauge position computation */
+gboolean lingot_mainframe_callback_gauge_computation(gpointer data) {
 	unsigned int period;
+	const FLT Log2 = log(2.0);
+	double fret_f;
+	GtkWidget* message_dialog;
 	LingotMainFrame* frame = (LingotMainFrame*) data;
+	char* error_message = lingot_error_queue_pop();
 
-	if (frame->core->running) {
-		lingot_mainframe_put_frequency(frame);
+	if (error_message != NULL) {
+		message_dialog = gtk_message_dialog_new(NULL,
+				GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_CLOSE, error_message);
+		gtk_window_set_title(GTK_WINDOW(message_dialog), _("Error"));
+		gtk_window_set_icon(GTK_WINDOW(message_dialog), gtk_window_get_icon(
+				GTK_WINDOW(frame->win)));
+		gtk_dialog_run(GTK_DIALOG(message_dialog));
+		gtk_widget_destroy(message_dialog);
+		free(error_message);
+	}
 
-		period = 1000/GAUGE_RATE;
-		frame->freq_timer_uid = g_timeout_add(period,
-				lingot_mainframe_callback_frequency, frame);
+	period = 1000 / GAUGE_RATE;
+	frame->freq_timer_uid = g_timeout_add(period,
+			lingot_mainframe_callback_gauge_computation, frame);
+
+	if (!frame->core->running || isnan(frame->core->freq) || (frame->core->freq
+			< 10.0)) {
+		lingot_gauge_compute(frame->gauge, frame->conf->vr);
+	} else {
+		// bring up some octaves to avoid negative frets.
+		fret_f = log(frame->core->freq / frame->conf->root_frequency) / Log2
+				* 12.0 + 12e2;
+		lingot_gauge_compute(frame->gauge, fret_f - rint(fret_f));
 	}
 
 	return 0;
@@ -208,18 +221,17 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 	frame->conf = lingot_config_new();
 	lingot_config_load(frame->conf, CONFIG_FILE_NAME);
 
-	frame->gauge = lingot_gauge_new(frame->conf->vr); // gauge in rest situation	
 	frame->core = lingot_core_new(frame->conf);
 
+	frame->gauge = lingot_gauge_new(frame->conf->vr); // gauge in rest situation
+
 	// ----- FREQUENCY FILTER CONFIGURATION ------
 
 	// low pass IIR filter.
-	FLT freq_filter_a[] =
-	{	1.0, -0.5};
-	FLT freq_filter_b[] =
-	{	0.5};
+	FLT freq_filter_a[] = { 1.0, -0.5 };
+	FLT freq_filter_b[] = { 0.5 };
 
-	frame->freq_filter= lingot_filter_new( 1, 0, freq_filter_a, freq_filter_b);
+	frame->freq_filter = lingot_filter_new(1, 0, freq_filter_a, freq_filter_b);
 
 	// ---------------------------------------------------
 
@@ -260,7 +272,7 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 	quit_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, accel_group);
 	about_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT,
 			accel_group);
-	view_spectrum_item= gtk_check_menu_item_new_with_label(_("Spectrum"));
+	view_spectrum_item = gtk_check_menu_item_new_with_label(_("Spectrum"));
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(view_spectrum_item),
 			TRUE);
 
@@ -281,11 +293,11 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 	gtk_signal_connect(GTK_OBJECT(preferences_item), "activate",
 			GTK_SIGNAL_FUNC(lingot_mainframe_callback_config_dialog), frame);
 
-	gtk_signal_connect(GTK_OBJECT(quit_item), "activate",
-			GTK_SIGNAL_FUNC(lingot_mainframe_callback_destroy), frame);
+	gtk_signal_connect(GTK_OBJECT(quit_item), "activate", GTK_SIGNAL_FUNC(
+			lingot_mainframe_callback_destroy), frame);
 
-	gtk_signal_connect(GTK_OBJECT(about_item), "activate",
-			GTK_SIGNAL_FUNC(lingot_mainframe_callback_about), frame);
+	gtk_signal_connect(GTK_OBJECT(about_item), "activate", GTK_SIGNAL_FUNC(
+			lingot_mainframe_callback_about), frame);
 
 	gtk_signal_connect(GTK_OBJECT(view_spectrum_item), "activate",
 			GTK_SIGNAL_FUNC(lingot_mainframe_callback_view_spectrum), frame);
@@ -325,8 +337,8 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 	//  gtk_fixed_put(GTK_FIXED(fix), frame1, 0, 0);
 	gtk_box_pack_start(GTK_BOX(horizontal_box), frame1, FALSE, FALSE, 0);
 
-	// note frame
-	frame3 = gtk_frame_new(_("Note"));
+	// tone frame
+	frame3 = gtk_frame_new(_("Tone"));
 	//gtk_fixed_put(GTK_BOX(hb), frame3, 164, 0);
 	gtk_box_pack_start(GTK_BOX(horizontal_box), frame3, TRUE, TRUE, 0);
 
@@ -342,14 +354,14 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 
 	// for spectrum drawing
 	GtkObject* adjust = gtk_adjustment_new(0.0, 0.0, ((frame->conf->fft_size
-			> 256) ? 0.5 : 1.0)*frame->conf->sample_rate, 1.0, 100.0, 100.0);
+			> 256) ? 0.5 : 1.0) * frame->conf->sample_rate, 1.0, 100.0, 100.0);
 	frame->spectrum_scroll = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(
 			GTK_ADJUSTMENT(adjust), NULL));
 	frame->spectrum_area = gtk_drawing_area_new();
 
 	int
 			x = ((frame->conf->fft_size > 256) ? (frame->conf->fft_size >> 1)
-					: 256) + 2*spectrum_x_margin;
+					: 256) + 2 * spectrum_x_margin;
 	int y = spectrum_size_y + spectrum_bottom_margin + spectrum_top_margin;
 
 	gtk_widget_set_size_request(GTK_WIDGET(frame->spectrum_area), x, y);
@@ -359,15 +371,15 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 	gtk_scrolled_window_add_with_viewport(frame->spectrum_scroll,
 			frame->spectrum_area);
 	gtk_widget_set_size_request(GTK_WIDGET(frame->spectrum_scroll), 260 + 2
-			*spectrum_x_margin, spectrum_size_y + spectrum_bottom_margin
+			* spectrum_x_margin, spectrum_size_y + spectrum_bottom_margin
 			+ spectrum_top_margin + 4
 			+ ((frame->conf->fft_size > 512) ? 16 : 0));
-	gtk_container_add(GTK_CONTAINER(spectrum_frame),
-			GTK_WIDGET(frame->spectrum_scroll));
+	gtk_container_add(GTK_CONTAINER(spectrum_frame), GTK_WIDGET(
+			frame->spectrum_scroll));
 
-	// for note and frequency displaying
+	// for tone and frequency displaying
 	vbinfo = gtk_vbox_new(FALSE, 0);
-	gtk_widget_set_size_request(GTK_WIDGET(vbinfo), 96 + 2*spectrum_x_margin,
+	gtk_widget_set_size_request(GTK_WIDGET(vbinfo), 96 + 2 * spectrum_x_margin,
 			gauge_size_y);
 	gtk_container_add(GTK_CONTAINER(frame3), vbinfo);
 
@@ -377,37 +389,38 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 	frame->error_label = gtk_label_new(_("err"));
 	gtk_box_pack_end_defaults(GTK_BOX(vbinfo), frame->error_label);
 
-	frame->note_label = gtk_label_new(_("note"));
-	gtk_widget_set_name(frame->note_label, "label_nota");
-	gtk_box_pack_end_defaults(GTK_BOX(vbinfo), frame->note_label);
+	frame->tone_label = gtk_label_new("");
+	gtk_widget_set_name(frame->tone_label, "label_nota");
+	gtk_box_pack_end_defaults(GTK_BOX(vbinfo), frame->tone_label);
 
 	// show all
 	gtk_widget_show_all(frame->win);
 
-	// two pixmaps for double buffer in gauge and spectrum drawing 
+	// two pixmaps for double buffer in gauge and spectrum drawing
 	// (virtual screen)
 	gdk_pixmap_new(frame->gauge_area->window, gauge_size_x, gauge_size_y, -1);
-	frame->pix_spectrum= gdk_pixmap_new(frame->spectrum_area->window, x, y, -1);
+	frame->pix_spectrum
+			= gdk_pixmap_new(frame->spectrum_area->window, x, y, -1);
 
 	// GTK signals
 	gtk_signal_connect(GTK_OBJECT(frame->gauge_area), "expose_event",
 			GTK_SIGNAL_FUNC(lingot_mainframe_callback_redraw), frame);
 	gtk_signal_connect(GTK_OBJECT(frame->spectrum_area), "expose_event",
 			GTK_SIGNAL_FUNC(lingot_mainframe_callback_redraw), frame);
-	gtk_signal_connect(GTK_OBJECT(frame->win), "destroy",
-			GTK_SIGNAL_FUNC(lingot_mainframe_callback_destroy), frame);
+	gtk_signal_connect(GTK_OBJECT(frame->win), "destroy", GTK_SIGNAL_FUNC(
+			lingot_mainframe_callback_destroy), frame);
 
-	period = 1000/frame->conf->visualization_rate;
+	period = 1000 / frame->conf->visualization_rate;
 	frame->visualization_timer_uid = g_timeout_add(period,
-			lingot_mainframe_callback_visualization, frame);
+			lingot_mainframe_callback_tout_gauge_display, frame);
 
-	period = 1000/frame->conf->calculation_rate;
+	period = 1000 / frame->conf->calculation_rate;
 	frame->calculation_timer_uid = g_timeout_add(period,
-			lingot_mainframe_callback_calculation, frame);
+			lingot_mainframe_callback_tout_spectrum_computation_display, frame);
 
-	period = 1000/GAUGE_RATE;
+	period = 1000 / GAUGE_RATE;
 	frame->freq_timer_uid = g_timeout_add(period,
-			lingot_mainframe_callback_frequency, frame);
+			lingot_mainframe_callback_gauge_computation, frame);
 
 	lingot_mainframe_color(&gauge_color, 0xC000, 0x0000, 0x2000);
 	lingot_mainframe_color(&spectrum_background_color, 0x1111, 0x3333, 0x1111);
@@ -424,8 +437,8 @@ LingotMainFrame* lingot_mainframe_new(int argc, char *argv[]) {
 	gdk_color_alloc(gdk_colormap_get_system(), &freq_color);
 	gdk_color_black(gdk_colormap_get_system(), &black_color);
 
-	spectrum_legend_font_desc
-			=pango_font_description_from_string("Helvetica Plain 7");
+	spectrum_legend_font_desc = pango_font_description_from_string(
+			"Helvetica Plain 7");
 
 	return frame;
 }
@@ -441,7 +454,7 @@ void lingot_mainframe_destroy(LingotMainFrame* frame) {
 	// pango_font_description_free(spectrum_legend_font_desc);
 	// gtk_widget_destroy(frame->freq_label);
 	// gtk_widget_destroy(frame->error_label);
-	// gtk_widget_destroy(frame->note_label);
+	// gtk_widget_destroy(frame->tone_label);
 	// gtk_widget_destroy(frame->win);
 
 	// gdk_color_free(&gauge_color);
@@ -463,13 +476,13 @@ void lingot_mainframe_run(LingotMainFrame* frame) {
 // ---------------------------------------------------------------------------
 
 void lingot_mainframe_redraw(LingotMainFrame* frame) {
-	lingot_mainframe_draw_gauge(frame);
+	lingot_mainframe_draw_gauge_and_labels(frame);
 	lingot_mainframe_draw_spectrum(frame);
 }
 
 // ---------------------------------------------------------------------------
 
-void lingot_mainframe_draw_gauge(LingotMainFrame* frame) {
+void lingot_mainframe_draw_gauge_and_labels(LingotMainFrame* frame) {
 	GdkGC* gc = frame->gauge_area->style->fg_gc[frame->gauge_area->state];
 	GdkWindow* w = frame->gauge_area->window;
 	GdkGCValues gv;
@@ -482,8 +495,7 @@ void lingot_mainframe_draw_gauge(LingotMainFrame* frame) {
 	// draws background
 	if (!frame->pix_stick) {
 		frame->pix_stick = gdk_pixmap_create_from_xpm_d(
-				frame->gauge_area->window, NULL, 
-				NULL, background2_xpm);
+				frame->gauge_area->window, NULL, NULL, background2_xpm);
 	}
 	gdk_draw_pixmap(frame->gauge_area->window, gc, frame->pix_stick, 0, 0, 0,
 			0, 160, 100);
@@ -491,13 +503,12 @@ void lingot_mainframe_draw_gauge(LingotMainFrame* frame) {
 	// and draws gauge
 	gdk_gc_set_foreground(gc, &gauge_color);
 
-	gdk_draw_line(w, gc, gauge_size_x >> 1, gauge_size_y - 1,
-			(gauge_size_x>> 1) + (int)rint(gauge_size
-					*sin(frame->gauge->position *M_PI/(1.5*max))), gauge_size_y
-					- 1- (int)rint(gauge_size *cos(frame->gauge->position*M_PI
-					/(1.5*max))));
+	gdk_draw_line(w, gc, gauge_size_x >> 1, gauge_size_y - 1, (gauge_size_x
+			>> 1) + (int) rint(gauge_size * sin(frame->gauge->position * M_PI
+			/ (1.5 * max))), gauge_size_y - 1 - (int) rint(gauge_size * cos(
+			frame->gauge->position * M_PI / (1.5 * max))));
 
-	// black edge.  
+	// black edge.
 	gdk_gc_set_foreground(gc, &black_color);
 	gdk_draw_rectangle(w, gc, FALSE, 0, 0, gauge_size_x - 1, gauge_size_y - 1);
 
@@ -507,6 +518,18 @@ void lingot_mainframe_draw_gauge(LingotMainFrame* frame) {
 }
 
 void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
+	// tone indexation by semitone.
+	static char* tone_string[] = { "A", "A#", "B", "C", "C#", "D", "D#", "E",
+			"F", "F#", "G", "G#" };
+	const FLT Log2 = log(2.0);
+
+	FLT fret_f;
+	int fret;
+	char* current_tone;
+	GtkWidget* widget = NULL;
+
+	static char error_string[30], freq_string[30];
+
 	PangoLayout* layout;
 
 	int
@@ -517,8 +540,8 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 	// minimum grid size in pixels
 	static int minimum_grid_width = 50;
 
-	/* scale factors to draw the grid. We will choose the smaller factor that
-	 respects the minimum_grid_width */
+	// scale factors (in KHz) to draw the grid. We will choose the smaller
+	// factor that respects the minimum_grid_width
 	static double
 			scales[] = { 0.01, 0.05, 0.1, 0.2, 0.5, 1, 2, 4, 11, 22, -1.0 };
 
@@ -533,26 +556,27 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 	int old_j;
 
 	GdkGC* gc = frame->spectrum_area->style->fg_gc[frame->spectrum_area->state];
-	GdkWindow* window = frame->pix_spectrum; //spectrum->window;
+	GdkPixmap* pixmap = frame->pix_spectrum; //spectrum->window;
 	GdkGCValues gv;
 	gdk_gc_get_values(gc, &gv);
 
 	// clear all
 	gdk_gc_set_foreground(gc, &spectrum_background_color);
-	gdk_draw_rectangle(window, gc, TRUE, 0, 0, spectrum_size_x + 2
-			*spectrum_x_margin, spectrum_size_y + spectrum_bottom_margin
+	gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, spectrum_size_x + 2
+			* spectrum_x_margin, spectrum_size_y + spectrum_bottom_margin
 			+ spectrum_top_margin);
 
 	gdk_gc_set_foreground(gc, &grid_color);
 
-	gdk_draw_line(window, gc, spectrum_x_margin, spectrum_size_y
+	gdk_draw_line(pixmap, gc, spectrum_x_margin, spectrum_size_y
 			+ spectrum_top_margin, spectrum_x_margin + spectrum_size_x,
 			spectrum_size_y + spectrum_top_margin);
 
 	// choose scale factor
 	for (i = 0; scales[i] > 0.0; i++) {
-		if ((1e3*scales[i]*frame->conf->fft_size*frame->conf->oversampling
-				/frame->conf->sample_rate) > minimum_grid_width)
+		if ((1e3 * scales[i] * frame->conf->fft_size
+				* frame->conf->oversampling / frame->conf->sample_rate)
+				> minimum_grid_width)
 			break;
 	}
 
@@ -561,14 +585,14 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 
 	FLT scale = scales[i];
 
-	int grid_width = 1e3*scales[i]*frame->conf->fft_size
-			*frame->conf->oversampling/frame->conf->sample_rate;
+	int grid_width = 1e3 * scales[i] * frame->conf->fft_size
+			* frame->conf->oversampling / frame->conf->sample_rate;
 
 	char buff[10];
 
 	FLT freq = 0.0;
 	for (i = 0; i <= spectrum_size_x; i += grid_width) {
-		gdk_draw_line(window, gc, spectrum_x_margin + i, spectrum_top_margin,
+		gdk_draw_line(pixmap, gc, spectrum_x_margin + i, spectrum_top_margin,
 				spectrum_x_margin + i, spectrum_size_y + spectrum_top_margin
 						+ 3);
 
@@ -576,22 +600,23 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 			sprintf(buff, "0 Hz");
 		} else if (floor(freq) == freq)
 			sprintf(buff, "%0.0f kHz", freq);
-		else if (floor(10*freq) == 10*freq) {
+		else if (floor(10 * freq) == 10 * freq) {
 			if (freq <= 1000.0)
-				sprintf(buff, "%0.0f Hz", 1e3*freq);
+				sprintf(buff, "%0.0f Hz", 1e3 * freq);
 			else
 				sprintf(buff, "%0.1f kHz", freq);
 		} else {
 			if (freq <= 100.0)
-				sprintf(buff, "%0.0f Hz", 1e3*freq);
+				sprintf(buff, "%0.0f Hz", 1e3 * freq);
 			else
 				sprintf(buff, "%0.2f kHz", freq);
 		}
 
 		layout = gtk_widget_create_pango_layout(frame->spectrum_area, buff);
 		pango_layout_set_font_description(layout, spectrum_legend_font_desc);
-		gdk_draw_layout(window, gc, spectrum_x_margin - 8 + i, spectrum_size_y
+		gdk_draw_layout(pixmap, gc, spectrum_x_margin - 8 + i, spectrum_size_y
 				+ spectrum_top_margin + 5, layout);
+		g_object_unref(layout);
 		freq += scale;
 	}
 
@@ -601,9 +626,11 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 
 	layout = gtk_widget_create_pango_layout(frame->spectrum_area, buff);
 	pango_layout_set_font_description(layout, spectrum_legend_font_desc);
-	gdk_draw_layout(window, gc, spectrum_x_margin - 6, 2, layout);
+	gdk_draw_layout(pixmap, gc, spectrum_x_margin - 6, 2, layout);
+	g_object_unref(layout);
 
-	int grid_height = (int) (PLOT_GAIN*log10(pow(10.0, grid_db_height/10.0))); // dB.
+	int grid_height = (int) (PLOT_GAIN
+			* log10(pow(10.0, grid_db_height / 10.0))); // dB.
 	j = 0;
 	for (i = 0; i <= spectrum_size_y; i += grid_height) {
 		if (j == 0)
@@ -613,10 +640,11 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 
 		layout = gtk_widget_create_pango_layout(frame->spectrum_area, buff);
 		pango_layout_set_font_description(layout, spectrum_legend_font_desc);
-		gdk_draw_layout(window, gc, 2, spectrum_size_y + spectrum_top_margin
+		gdk_draw_layout(pixmap, gc, 2, spectrum_size_y + spectrum_top_margin
 				- i - 5, layout);
+		g_object_unref(layout);
 
-		gdk_draw_line(window, gc, spectrum_x_margin, spectrum_size_y
+		gdk_draw_line(pixmap, gc, spectrum_x_margin, spectrum_size_y
 				+ spectrum_top_margin - i, spectrum_x_margin + spectrum_size_x,
 				spectrum_size_y + spectrum_top_margin - i);
 
@@ -631,15 +659,12 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 		if ((i % 10) > 5)
 			continue;
 
-		FLT w = 2*M_PI*i/frame->conf->fft_size;
-		//FLT noise = pow(10.0, (frame->conf->noise_threshold_db*(1.0 - 0.9*w/M_PI))/10.0);
-		//FLT noise = lingot_signal_get_noise_threshold(frame->conf, w);
 		FLT noise = frame->conf->noise_threshold_nu;
 		old_j = j;
-		j = (noise > 1.0) ? (int) (PLOT_GAIN*log10(noise)) : 0; // dB.
-		if ((old_j >= 0) && (old_j < spectrum_size_y)&& (j >= 0)&& (j
+		j = (noise > 1.0) ? (int) (PLOT_GAIN * log10(noise)) : 0; // dB.
+		if ((old_j >= 0) && (old_j < spectrum_size_y) && (j >= 0) && (j
 				< spectrum_size_y))
-			gdk_draw_line(window, gc, spectrum_x_margin + i - 1,
+			gdk_draw_line(pixmap, gc, spectrum_x_margin + i - 1,
 					spectrum_size_y + spectrum_top_margin - old_j,
 					spectrum_x_margin + i, spectrum_size_y
 							+ spectrum_top_margin - j);
@@ -648,37 +673,55 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 	gdk_gc_set_foreground(gc, &spectrum_color);
 
 	// spectrum drawing.
-	j = -1;
-	for (i = 0; (i < frame->conf->fft_size - 1) && (i < spectrum_size_x - 1); i++) {
-		old_j = j;
-		j = (frame->core->X[i] > 1.0) ? (int) (PLOT_GAIN*log10(frame->core->X[i])) : 0; // dB.
-		if (spectrum_drawing_filled) {
-			if (j < spectrum_size_y)
-				gdk_draw_line(window, gc, spectrum_x_margin + i,
+	if (frame->core->running) {
+		j = -1;
+
+		GtkAdjustment* adj = gtk_scrolled_window_get_hadjustment(
+				frame->spectrum_scroll);
+		int min = gtk_adjustment_get_value(adj) - spectrum_x_margin;
+		int max = min + 256 + 3 * spectrum_x_margin;
+		if (min < 0)
+			min = 0;
+		if (max >= spectrum_size_x)
+			max = spectrum_size_x;
+
+		for (i = min; i < max; i++) {
+			j = (frame->core->X[i] > 1.0) ? (int) (PLOT_GAIN * log10(
+					frame->core->X[i])) : 0; // dB.
+			if (j >= spectrum_size_y)
+				j = spectrum_size_y - 1;
+			if (spectrum_drawing_filled) {
+				gdk_draw_line(pixmap, gc, spectrum_x_margin + i,
 						spectrum_size_y + spectrum_top_margin - 1,
-						spectrum_x_margin + i, spectrum_top_margin + ((j
-								< spectrum_size_y) ? (spectrum_size_y - j) : 0));
-		} else if ((old_j >= 0) && (old_j < spectrum_size_y)&& (j >= 0)&& (j
-				< spectrum_size_y))
-			gdk_draw_line(window, gc, spectrum_x_margin + i - 1,
-					spectrum_size_y + spectrum_top_margin - old_j,
-					spectrum_x_margin + i, spectrum_size_y
-							+ spectrum_top_margin - j);
-	}
+						spectrum_x_margin + i, spectrum_top_margin
+								+ spectrum_size_y - j);
+			} else {
+				if ((old_j >= 0) && (old_j < spectrum_size_y) && (j >= 0) && (j
+						< spectrum_size_y))
+					gdk_draw_line(pixmap, gc, spectrum_x_margin + i - 1,
+							spectrum_size_y + spectrum_top_margin - old_j,
+							spectrum_x_margin + i, spectrum_size_y
+									+ spectrum_top_margin - j);
+				old_j = j;
+			}
+		}
 
-	if (frame->core->freq != 0.0) {
+		if (frame->core->freq != 0.0) {
 
-		// fundamental frequency mark with a red point. 
-		gdk_gc_set_foreground(gc, &freq_color);
+			// fundamental frequency mark with a red point.
+			gdk_gc_set_foreground(gc, &freq_color);
 
-		// index of closest sample to fundamental frequency.
-		i = (int) rint(frame->core->freq*frame->conf->fft_size
-				*frame->conf->oversampling/frame->conf->sample_rate);
-		if ((i < frame->conf->fft_size - 1) && (i < spectrum_size_x - 1)) {
-			j = (frame->core->X[i] > 1.0) ? (int) (PLOT_GAIN*log10(frame->core->X[i])) : 0; // dB.
-			if (j < spectrum_size_y - 1)
-				gdk_draw_rectangle(window, gc, TRUE, spectrum_x_margin + i-1,
-						spectrum_size_y + spectrum_top_margin - j - 1, 3, 3);
+			// index of closest sample to fundamental frequency.
+			i = (int) rint(frame->core->freq * frame->conf->fft_size
+					* frame->conf->oversampling / frame->conf->sample_rate);
+			if ((i < frame->conf->fft_size - 1) && (i < spectrum_size_x - 1)) {
+				j = (frame->core->X[i] > 1.0) ? (int) (PLOT_GAIN * log10(
+						frame->core->X[i])) : 0; // dB.
+				if (j < spectrum_size_y - 1)
+					gdk_draw_rectangle(pixmap, gc, TRUE, spectrum_x_margin + i
+							- 1, spectrum_size_y + spectrum_top_margin - j - 1,
+							3, 3);
+			}
 		}
 	}
 
@@ -686,43 +729,35 @@ void lingot_mainframe_draw_spectrum(LingotMainFrame* frame) {
 
 	gdk_gc_set_foreground(gc, &black_color);
 
-	gdk_draw_pixmap(frame->spectrum_area->window, gc, window, 0, 0, 0, 0,
-			spectrum_size_x + 2*spectrum_x_margin, spectrum_size_y
-					+ spectrum_bottom_margin + spectrum_top_margin);
-	gdk_flush();
-}
+	widget = frame->spectrum_area;
 
-void lingot_mainframe_put_frequency(LingotMainFrame* frame) {
-	// note indexation by semitone.
-	static char* note_string[] = { "A", "A#", "B", "C", "C#", "D", "D#", "E",
-			"F", "F#", "G", "G#" };
-	const FLT Log2 = log(2.0);
+	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(
+			widget)], pixmap, 0, 0, 0, 0, spectrum_size_x + 2
+			* spectrum_x_margin, spectrum_size_y + spectrum_bottom_margin
+			+ spectrum_top_margin);
 
-	FLT fret_f;
-	int fret;
-	char* current_note;
-	char labeltext_current_note[100];
+	//gdk_flush();
 
-	static char error_string[30], freq_string[30];
-
-	LingotCore* core1 = frame->core;
+	// draw note, error and frequency labels
 
 	// ignore continuous component
-	if (isnan(frame->core->freq) || (frame->core->freq < 10.0)) {
-		current_note = "---";
+	if (!frame->core->running || isnan(frame->core->freq) || (frame->core->freq
+			< 10.0)) {
+		current_tone = "---";
 		strcpy(error_string, "e = ---");
 		strcpy(freq_string, "f = ---");
-		lingot_gauge_compute(frame->gauge, frame->conf->vr);
+		//lingot_gauge_compute(frame->gauge, frame->conf->vr);
 	} else {
 		// bring up some octaves to avoid negative frets.
-		fret_f = log(frame->core->freq/frame->conf->root_frequency)/Log2*12.0
-				+ 12e2;
+		fret_f = log(frame->core->freq / frame->conf->root_frequency) / Log2
+				* 12.0 + 12e2;
 
-		lingot_gauge_compute(frame->gauge, fret_f - rint(fret_f));
+		//lingot_gauge_compute(frame->gauge, fret_f - rint(fret_f));
 		fret = ((int) rint(fret_f)) % 12;
 
-		current_note = note_string[fret];
-		sprintf(error_string, "e = %+4.2f%%", frame->gauge->position*100.0);
+		current_tone = tone_string[fret];
+		sprintf(error_string, "e = %+2.0f cents", (frame->gauge->position
+				* 100.0));
 		sprintf(freq_string, "f = %6.2f Hz", lingot_filter_filter_sample(
 				frame->freq_filter, frame->core->freq));
 	}
@@ -731,8 +766,8 @@ void lingot_mainframe_put_frequency(LingotMainFrame* frame) {
 	gtk_label_set_text(GTK_LABEL(frame->error_label), error_string);
 
 	char* markup = g_markup_printf_escaped(
-			"<span size=\"xx-large\" weight=\"bold\">%s</span>", current_note);
-	gtk_label_set_markup(GTK_LABEL(frame->note_label), markup);
+			"<span size=\"xx-large\" weight=\"bold\">%s</span>", current_tone);
+	gtk_label_set_markup(GTK_LABEL(frame->tone_label), markup);
 	g_free(markup);
 }
 
@@ -748,16 +783,17 @@ void lingot_mainframe_change_config(LingotMainFrame* frame, LingotConfig* conf)
 
 	int
 			x = ((frame->conf->fft_size > 256) ? (frame->conf->fft_size >> 1)
-					: 256) + 2*spectrum_x_margin;
+					: 256) + 2 * spectrum_x_margin;
 	int y = spectrum_size_y + spectrum_top_margin + spectrum_bottom_margin;
 	gtk_widget_set_size_request(GTK_WIDGET(frame->spectrum_area), x, y);
-	frame->pix_spectrum= gdk_pixmap_new(frame->spectrum_area->window, x, y, -1);
+	frame->pix_spectrum
+			= gdk_pixmap_new(frame->spectrum_area->window, x, y, -1);
 
 	gtk_scrolled_window_set_policy(frame->spectrum_scroll,
 			(frame->conf->fft_size > 512) ? GTK_POLICY_ALWAYS
 					: GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 	gtk_widget_set_size_request(GTK_WIDGET(frame->spectrum_scroll), 260 + 2
-			*spectrum_x_margin, spectrum_size_y + spectrum_bottom_margin
+			* spectrum_x_margin, spectrum_size_y + spectrum_bottom_margin
 			+ spectrum_top_margin + 4
 			+ ((frame->conf->fft_size > 512) ? 16 : 0));
 
